@@ -21,25 +21,21 @@
 /**
  * SECTION:element-dvdec
  *
- * <refsect2>
- * <para>
  * dvdec decodes DV video into raw video. The element expects a full DV frame
  * as input, which is 120000 bytes for NTSC and 144000 for PAL video.
- * </para>
- * <para>
- * This element can perform simple frame dropping with the drop-factor
+ *
+ * This element can perform simple frame dropping with the #GstDVDec:drop-factor
  * property. Setting this property to a value N > 1 will only decode every 
  * Nth frame.
- * </para>
+ *
+ * <refsect2>
  * <title>Example launch line</title>
- * <para>
- * <programlisting>
+ * |[
  * gst-launch filesrc location=test.dv ! dvdemux name=demux ! dvdec ! xvimagesink
- * </programlisting>
- * This pipeline decodes and renders the raw DV stream to a videosink.
- * </para>
- * Last reviewed on 2006-02-28 (0.10.3)
+ * ]| This pipeline decodes and renders the raw DV stream to a videosink.
  * </refsect2>
+ *
+ * Last reviewed on 2006-02-28 (0.10.3)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -47,6 +43,7 @@
 #endif
 #include <string.h>
 #include <math.h>
+#include <gst/video/video.h>
 
 #include "gstdvdec.h"
 
@@ -213,9 +210,6 @@ gst_dvdec_class_init (GstDVDecClass * klass)
           1, G_MAXINT, DV_DEFAULT_DECODE_NTH, G_PARAM_READWRITE));
 
   gstelement_class->change_state = gst_dvdec_change_state;
-
-  /* table initialization, only do once */
-  dv_init (0, 0);
 }
 
 static void
@@ -335,7 +329,7 @@ gst_dvdec_src_negotiate (GstDVDec * dvdec)
       "framerate", GST_TYPE_FRACTION, dvdec->framerate_numerator,
       dvdec->framerate_denominator,
       "pixel-aspect-ratio", GST_TYPE_FRACTION, dvdec->par_x,
-      dvdec->par_y, NULL);
+      dvdec->par_y, "interlaced", G_TYPE_BOOLEAN, dvdec->interlaced, NULL);
 
   gst_pad_set_caps (dvdec->srcpad, othercaps);
   gst_caps_unref (othercaps);
@@ -441,6 +435,7 @@ gst_dvdec_chain (GstPad * pad, GstBuffer * buf)
 
   dvdec->height = (dvdec->PAL ? PAL_HEIGHT : NTSC_HEIGHT);
 
+  dvdec->interlaced = !dv_is_progressive (dvdec->decoder);
 
   /* negotiate if not done yet */
   if (!dvdec->src_negotiated) {
@@ -473,6 +468,8 @@ gst_dvdec_chain (GstPad * pad, GstBuffer * buf)
   dv_decode_full_frame (dvdec->decoder, inframe,
       e_dv_color_yuv, outframe_ptrs, outframe_pitches);
 
+  GST_BUFFER_FLAG_UNSET (outbuf, GST_VIDEO_BUFFER_TFF);
+
   GST_BUFFER_OFFSET (outbuf) = GST_BUFFER_OFFSET (buf);
   GST_BUFFER_OFFSET_END (outbuf) = GST_BUFFER_OFFSET_END (buf);
   GST_BUFFER_TIMESTAMP (outbuf) = cstart;
