@@ -16,6 +16,23 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+/**
+ * SECTION:element-cacasink
+ * @see_also: #GstAASink
+ *
+ * Displays video as color ascii art.
+ *
+ * <refsect2>
+ * <title>Example launch line</title>
+ * |[
+ * CACA_GEOMETRY=160x60 CACA_FONT=5x7 gst-launch filesrc location=test.avi ! decodebin ! ffmpegcolorspace ! cacasink
+ * ]| This pipeline renders a video to ascii art into a separate window using a
+ * small font and specifying the ascii resolution.
+ * |[
+ * CACA_DRIVER=ncurses gst-launch filesrc location=test.avi ! decodebin ! ffmpegcolorspace ! cacasink
+ * ]| This pipeline renders a video to ascii art into the current terminal.
+ * </refsect2>
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -26,6 +43,17 @@
 
 #include "gstcacasink.h"
 
+#define GST_CACA_DEFAULT_SCREEN_WIDTH 80
+#define GST_CACA_DEFAULT_SCREEN_HEIGHT 25
+#define GST_CACA_DEFAULT_BPP 24
+#define GST_CACA_DEFAULT_RED_MASK GST_VIDEO_BYTE1_MASK_32_INT
+#define GST_CACA_DEFAULT_GREEN_MASK GST_VIDEO_BYTE2_MASK_32_INT
+#define GST_CACA_DEFAULT_BLUE_MASK GST_VIDEO_BYTE3_MASK_32_INT
+
+//#define GST_CACA_DEFAULT_RED_MASK R_MASK_32_REVERSE_INT
+//#define GST_CACA_DEFAULT_GREEN_MASK G_MASK_32_REVERSE_INT
+//#define GST_CACA_DEFAULT_BLUE_MASK B_MASK_32_REVERSE_INT
+
 /* elementfactory information */
 static const GstElementDetails gst_cacasink_details =
 GST_ELEMENT_DETAILS ("A colored ASCII art video sink",
@@ -233,11 +261,6 @@ gst_cacasink_setcaps (GstBaseSink * basesink, GstCaps * caps)
 static void
 gst_cacasink_init (GstCACASink * cacasink)
 {
-  GstPad *pad;
-
-  pad = GST_BASE_SINK_PAD (cacasink);
-  /* gst_pad_set_fixatecaps_function (pad, gst_cacasink_fixate); */
-
   cacasink->screen_width = GST_CACA_DEFAULT_SCREEN_WIDTH;
   cacasink->screen_height = GST_CACA_DEFAULT_SCREEN_HEIGHT;
   cacasink->bpp = GST_CACA_DEFAULT_BPP;
@@ -281,9 +304,7 @@ gst_cacasink_set_property (GObject * object, guint prop_id,
       cacasink->antialiasing = g_value_get_boolean (value);
       if (cacasink->antialiasing) {
         caca_set_feature (CACA_ANTIALIASING_MAX);
-      }
-
-      else {
+      } else {
         caca_set_feature (CACA_ANTIALIASING_MIN);
       }
       break;
@@ -329,7 +350,12 @@ static gboolean
 gst_cacasink_open (GstCACASink * cacasink)
 {
   cacasink->bitmap = NULL;
-  caca_init ();
+
+  if (caca_init () < 0) {
+    GST_ELEMENT_ERROR (cacasink, RESOURCE, OPEN_WRITE, (NULL),
+        ("caca_init() failed"));
+    return FALSE;
+  }
 
   cacasink->screen_width = caca_get_width ();
   cacasink->screen_height = caca_get_height ();
@@ -356,7 +382,6 @@ gst_cacasink_change_state (GstElement * element, GstStateChange transition)
 {
   GstStateChangeReturn ret;
 
-
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       if (!gst_cacasink_open (GST_CACASINK (element)))
