@@ -12,7 +12,6 @@
  * Boston, MA 02111-1307, USA.
  *
  */
-
 /**
  * SECTION:element-pngdec
  *
@@ -118,10 +117,8 @@ gst_pngdec_base_init (gpointer g_class)
 static void
 gst_pngdec_class_init (GstPngDecClass * klass)
 {
-  GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
 
-  gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
   parent_class = g_type_class_peek_parent (klass);
@@ -204,7 +201,14 @@ user_info_callback (png_structp png_ptr, png_infop info)
 
   /* Allocate output buffer */
   pngdec->rowbytes = png_get_rowbytes (pngdec->png, pngdec->info);
-  buffer_size = pngdec->height * GST_ROUND_UP_4 (pngdec->rowbytes);
+  if (pngdec->rowbytes > (G_MAXUINT32 - 3)
+      || pngdec->height > G_MAXUINT32 / pngdec->rowbytes) {
+    ret = GST_FLOW_ERROR;
+    goto beach;
+  }
+  pngdec->rowbytes = GST_ROUND_UP_4 (pngdec->rowbytes);
+  buffer_size = pngdec->height * pngdec->rowbytes;
+
   ret =
       gst_pad_alloc_buffer_and_set_caps (pngdec->srcpad, GST_BUFFER_OFFSET_NONE,
       buffer_size, GST_PAD_CAPS (pngdec->srcpad), &buffer);
@@ -228,17 +232,16 @@ user_endrow_callback (png_structp png_ptr, png_bytep new_row,
 
   /* FIXME: implement interlaced pictures */
 
+  /* If buffer_out doesn't exist, it means buffer_alloc failed, which 
+   * will already have set the return code */
   if (GST_IS_BUFFER (pngdec->buffer_out)) {
-    size_t offset = row_num * GST_ROUND_UP_4 (pngdec->rowbytes);
+    size_t offset = row_num * pngdec->rowbytes;
 
     GST_LOG ("got row %u, copying in buffer %p at offset %" G_GSIZE_FORMAT,
         (guint) row_num, pngdec->buffer_out, offset);
     memcpy (GST_BUFFER_DATA (pngdec->buffer_out) + offset, new_row,
         pngdec->rowbytes);
     pngdec->ret = GST_FLOW_OK;
-  } else {
-    GST_LOG ("we don't have any output buffer to write this row !");
-    pngdec->ret = GST_FLOW_ERROR;
   }
 }
 
@@ -248,15 +251,17 @@ buffer_clip (GstPngDec * dec, GstBuffer * buffer)
   gboolean res = TRUE;
   gint64 cstart, cstop;
 
+
   if ((!GST_CLOCK_TIME_IS_VALID (GST_BUFFER_TIMESTAMP (buffer))) ||
       (!GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DURATION (buffer))) ||
       (dec->segment.format != GST_FORMAT_TIME))
     goto beach;
 
+  cstart = GST_BUFFER_TIMESTAMP (buffer);
+  cstop = GST_BUFFER_DURATION (buffer);
+
   if ((res = gst_segment_clip (&dec->segment, GST_FORMAT_TIME,
-              GST_BUFFER_TIMESTAMP (buffer),
-              GST_BUFFER_TIMESTAMP (buffer) + GST_BUFFER_DURATION (buffer),
-              &cstart, &cstop))) {
+              cstart, cstart + cstop, &cstart, &cstop))) {
     GST_BUFFER_TIMESTAMP (buffer) = cstart;
     GST_BUFFER_DURATION (buffer) = cstop - cstart;
   }
@@ -303,6 +308,7 @@ user_read_data (png_structp png_ptr, png_bytep data, png_size_t length)
   GstPngDec *pngdec;
   GstBuffer *buffer;
   GstFlowReturn ret = GST_FLOW_OK;
+  guint size;
 
   pngdec = GST_PNGDEC (png_ptr->io_ptr);
 
@@ -310,10 +316,15 @@ user_read_data (png_structp png_ptr, png_bytep data, png_size_t length)
       pngdec->offset);
 
   ret = gst_pad_pull_range (pngdec->sinkpad, pngdec->offset, length, &buffer);
-  if ((ret != GST_FLOW_OK) || (GST_BUFFER_SIZE (buffer) != length))
+  if (ret != GST_FLOW_OK)
     goto pause;
 
-  memcpy (data, GST_BUFFER_DATA (buffer), GST_BUFFER_SIZE (buffer));
+  size = GST_BUFFER_SIZE (buffer);
+
+  if (size != length)
+    goto short_buffer;
+
+  memcpy (data, GST_BUFFER_DATA (buffer), size);
 
   gst_buffer_unref (buffer);
 
@@ -321,14 +332,28 @@ user_read_data (png_structp png_ptr, png_bytep data, png_size_t length)
 
   return;
 
+  /* ERRORS */
 pause:
-  GST_INFO_OBJECT (pngdec, "pausing task, reason %s", gst_flow_get_name (ret));
-  gst_pad_pause_task (pngdec->sinkpad);
-  if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
-    gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
+  {
+    GST_INFO_OBJECT (pngdec, "pausing task, reason %s",
+        gst_flow_get_name (ret));
+    gst_pad_pause_task (pngdec->sinkpad);
+    if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
+      GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
+          (_("Internal data stream error.")),
+          ("stream stopped, reason %s", gst_flow_get_name (ret)));
+      gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
+    }
+    return;
+  }
+short_buffer:
+  {
+    gst_buffer_unref (buffer);
     GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
         (_("Internal data stream error.")),
-        ("stream stopped, reason %s", gst_flow_get_name (ret)));
+        ("Read %u, needed %" G_GSIZE_FORMAT "bytes", size, length));
+    ret = GST_FLOW_ERROR;
+    goto pause;
   }
 }
 
@@ -372,7 +397,11 @@ gst_pngdec_caps_create_and_set (GstPngDec * pngdec)
     png_set_gray_to_rgb (pngdec->png);
     if (bpc < 8) {              /* Convert to 8 bits */
       GST_LOG_OBJECT (pngdec, "converting grayscale image to 8 bits");
+#if PNG_LIBPNG_VER < 10400
       png_set_gray_1_2_4_to_8 (pngdec->png);
+#else
+      png_set_expand_gray_1_2_4_to_8 (pngdec->png);
+#endif
     }
   }
 
@@ -478,13 +507,17 @@ gst_pngdec_task (GstPad * pad)
 
   /* Allocate output buffer */
   rowbytes = png_get_rowbytes (pngdec->png, pngdec->info);
-  buffer_size = pngdec->height * GST_ROUND_UP_4 (rowbytes);
+  if (rowbytes > (G_MAXUINT32 - 3) || pngdec->height > G_MAXUINT32 / rowbytes) {
+    ret = GST_FLOW_ERROR;
+    goto pause;
+  }
+  rowbytes = GST_ROUND_UP_4 (rowbytes);
+  buffer_size = pngdec->height * rowbytes;
   ret =
       gst_pad_alloc_buffer_and_set_caps (pngdec->srcpad, GST_BUFFER_OFFSET_NONE,
       buffer_size, GST_PAD_CAPS (pngdec->srcpad), &buffer);
-  if (ret != GST_FLOW_OK) {
+  if (ret != GST_FLOW_OK)
     goto pause;
-  }
 
   rows = (png_bytep *) g_malloc (sizeof (png_bytep) * pngdec->height);
 
@@ -492,18 +525,17 @@ gst_pngdec_task (GstPad * pad)
 
   for (i = 0; i < pngdec->height; i++) {
     rows[i] = inp;
-    inp += GST_ROUND_UP_4 (rowbytes);
+    inp += rowbytes;
   }
 
   /* Read the actual picture */
   png_read_image (pngdec->png, rows);
-  free (rows);
+  g_free (rows);
 
   /* Push the raw RGB frame */
   ret = gst_pad_push (pngdec->srcpad, buffer);
-  if (ret != GST_FLOW_OK) {
+  if (ret != GST_FLOW_OK)
     goto pause;
-  }
 
   /* And we are done */
   gst_pad_pause_task (pngdec->sinkpad);
@@ -511,13 +543,16 @@ gst_pngdec_task (GstPad * pad)
   return;
 
 pause:
-  GST_INFO_OBJECT (pngdec, "pausing task, reason %s", gst_flow_get_name (ret));
-  gst_pad_pause_task (pngdec->sinkpad);
-  if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
-    GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
-        (_("Internal data stream error.")),
-        ("stream stopped, reason %s", gst_flow_get_name (ret)));
-    gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
+  {
+    GST_INFO_OBJECT (pngdec, "pausing task, reason %s",
+        gst_flow_get_name (ret));
+    gst_pad_pause_task (pngdec->sinkpad);
+    if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
+      GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
+          (_("Internal data stream error.")),
+          ("stream stopped, reason %s", gst_flow_get_name (ret)));
+      gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
+    }
   }
 }
 
@@ -837,10 +872,6 @@ setup_failed:
 static gboolean
 gst_pngdec_sink_activate_pull (GstPad * sinkpad, gboolean active)
 {
-  GstPngDec *pngdec;
-
-  pngdec = GST_PNGDEC (GST_OBJECT_PARENT (sinkpad));
-
   if (active) {
     return gst_pad_start_task (sinkpad, (GstTaskFunction) gst_pngdec_task,
         sinkpad);
