@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
+ * Copyright (C) <2005> Wim Taymans <wim.taymans@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -39,10 +39,10 @@ GST_DEBUG_CATEGORY_STATIC (rtpamrdepay_debug);
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_amrdepay_details =
-GST_ELEMENT_DETAILS ("RTP packet depayloader",
+GST_ELEMENT_DETAILS ("RTP AMR depayloader",
     "Codec/Depayloader/Network",
     "Extracts AMR or AMR-WB audio from RTP packets (RFC 3267)",
-    "Wim Taymans <wim@fluendo.com>");
+    "Wim Taymans <wim.taymans@gmail.com>");
 
 /* RtpAMRDepay signals and args */
 enum
@@ -137,16 +137,10 @@ gst_rtp_amr_depay_base_init (gpointer klass)
 static void
 gst_rtp_amr_depay_class_init (GstRtpAMRDepayClass * klass)
 {
-  GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
   GstBaseRTPDepayloadClass *gstbasertpdepayload_class;
 
-  gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
   gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gstbasertpdepayload_class->process = gst_rtp_amr_depay_process;
   gstbasertpdepayload_class->set_caps = gst_rtp_amr_depay_setcaps;
 
@@ -174,6 +168,7 @@ gst_rtp_amr_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   const gchar *params;
   const gchar *str, *type;
   gint clock_rate, need_clock_rate;
+  gboolean res;
 
   rtpamrdepay = GST_RTP_AMR_DEPAY (depayload);
 
@@ -183,11 +178,11 @@ gst_rtp_amr_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   if ((str = gst_structure_get_string (structure, "encoding-name"))) {
     if (strcmp (str, "AMR") == 0) {
       rtpamrdepay->mode = GST_RTP_AMR_DP_MODE_NB;
-      clock_rate = need_clock_rate = 8000;
+      need_clock_rate = 8000;
       type = "audio/AMR";
     } else if (strcmp (str, "AMR-WB") == 0) {
       rtpamrdepay->mode = GST_RTP_AMR_DP_MODE_WB;
-      clock_rate = need_clock_rate = 16000;
+      need_clock_rate = 16000;
       type = "audio/AMR-WB";
     } else
       goto invalid_mode;
@@ -235,7 +230,8 @@ gst_rtp_amr_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
     rtpamrdepay->channels = atoi (params);
   }
 
-  gst_structure_get_int (structure, "clock-rate", &clock_rate);
+  if (!gst_structure_get_int (structure, "clock-rate", &clock_rate))
+    clock_rate = need_clock_rate;
   depayload->clock_rate = clock_rate;
 
   /* we require 1 channel, 8000 Hz, octet aligned, no CRC,
@@ -254,13 +250,10 @@ gst_rtp_amr_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
   srccaps = gst_caps_new_simple (type,
       "channels", G_TYPE_INT, rtpamrdepay->channels,
       "rate", G_TYPE_INT, clock_rate, NULL);
-
-  gst_pad_set_caps (GST_BASE_RTP_DEPAYLOAD_SRCPAD (depayload), srccaps);
+  res = gst_pad_set_caps (GST_BASE_RTP_DEPAYLOAD_SRCPAD (depayload), srccaps);
   gst_caps_unref (srccaps);
 
-  rtpamrdepay->negotiated = TRUE;
-
-  return TRUE;
+  return res;
 
   /* ERRORS */
 invalid_mode:
@@ -275,6 +268,7 @@ static gint nb_frame_size[16] = {
   12, 13, 15, 17, 19, 20, 26, 31,
   5, -1, -1, -1, -1, -1, -1, 0
 };
+
 static gint wb_frame_size[16] = {
   17, 23, 32, 36, 40, 46, 50, 58,
   60, -1, -1, -1, -1, -1, -1, 0
@@ -290,19 +284,13 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
 
   rtpamrdepay = GST_RTP_AMR_DEPAY (depayload);
 
-  if (!rtpamrdepay->negotiated)
-    goto not_negotiated;
-
-  if (!gst_rtp_buffer_validate (buf))
-    goto invalid_packet;
-
   /* setup frame size pointer */
   if (rtpamrdepay->mode == GST_RTP_AMR_DP_MODE_NB)
     frame_size = nb_frame_size;
   else
     frame_size = wb_frame_size;
 
-  /* when we get here, 1 channel, 8000/16000 Hz, octet aligned, no CRC, 
+  /* when we get here, 1 channel, 8000/16000 Hz, octet aligned, no CRC,
    * no robust sorting, no interleaving data is to be depayloaded */
   {
     guint8 *payload, *p, *dp;
@@ -310,9 +298,7 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     gint i, num_packets, num_nonempty_packets;
     gint amr_len;
     gint ILL, ILP;
-    gboolean marker;
 
-    marker = gst_rtp_buffer_get_marker (buf);
     payload_len = gst_rtp_buffer_get_payload_len (buf);
 
     /* need at least 2 bytes for the header */
@@ -324,7 +310,7 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     /* depay CMR. The CMR is used by the sender to request
      * a new encoding mode.
      *
-     *  0 1 2 3 4 5 6 7 
+     *  0 1 2 3 4 5 6 7
      * +-+-+-+-+-+-+-+-+
      * | CMR   |R|R|R|R|
      * +-+-+-+-+-+-+-+-+
@@ -348,8 +334,8 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
         goto wrong_interleaving;
     }
 
-    /* 
-     *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 
+    /*
+     *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6
      * +-+-+-+-+-+-+-+-+..
      * |F|  FT   |Q|P|P| more FT..
      * +-+-+-+-+-+-+-+-+..
@@ -420,34 +406,18 @@ gst_rtp_amr_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
     /* we can set the duration because each packet is 20 milliseconds */
     GST_BUFFER_DURATION (outbuf) = num_packets * 20 * GST_MSECOND;
 
-    if (marker) {
-      /* marker bit marks a discont buffer */
+    if (gst_rtp_buffer_get_marker (buf)) {
+      /* marker bit marks a discont buffer after a talkspurt. */
       GST_DEBUG_OBJECT (depayload, "marker bit was set");
       GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
-      marker = FALSE;
     }
 
-    gst_buffer_set_caps (outbuf,
-        GST_PAD_CAPS (GST_BASE_RTP_DEPAYLOAD_SRCPAD (depayload)));
-
     GST_DEBUG_OBJECT (depayload, "pushing buffer of size %d",
         GST_BUFFER_SIZE (outbuf));
   }
   return outbuf;
 
   /* ERRORS */
-invalid_packet:
-  {
-    GST_ELEMENT_WARNING (rtpamrdepay, STREAM, DECODE,
-        (NULL), ("AMR RTP packet did not validate"));
-    goto bad_packet;
-  }
-not_negotiated:
-  {
-    GST_ELEMENT_ERROR (rtpamrdepay, STREAM, NOT_IMPLEMENTED,
-        (NULL), ("not negotiated"));
-    return NULL;
-  }
 too_small:
   {
     GST_ELEMENT_WARNING (rtpamrdepay, STREAM, DECODE,
